<?php
/* $Id: class_album.inc,v 1.1.2.17 2006/04/29 16:52:36 vhmauery Exp $ */

/*
Acidfree Photo Albums for Drupal
Copyright (C) 2005 Vernon Mauery

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/

/**
 * @file
 * The is the class implementation of the album.  All the acidfree elements
 * must be defined in this same manner, which makes creating a new class
 * relatively simple -- copy one of the existing classes and modify the 
 * relevant functions.
 */

function class_album_register() {
    $class = new stdClass();
    $class->class = 'album';
    $class->max_size = 0;
    $class->mime_ext = Array();
    $class->create = '_class_album_create';
    $class->update = '_class_album_update';
    $class->form = '_class_album_form';
    $class->destroy = '_class_album_destroy';
    $class->access = 'create acidfree albums';
    return $class;
}

function _class_album_create(&$node, &$file) {
    $node->thumb = '';
}

function _class_album_update(&$node) {
    if ($node->thumb == '0') {
        $node->thumb = '';
    }
    return "thumb='{$node->thumb}',";
}

function _class_album_form($op, &$node, $name_prefix='') {
    switch ($op) {
    case 'new':
        break;
    case 'preview':
        break;
    case 'update':
        $mythumb = (is_object($node->thumb) ? $node->thumb->fid : $node->thumb);
        $options['0'] = 'Random';
        $children = _acidfree_get_children($node, -1, 0, true);
        if (!is_array($children)) {
            if (count($children) == 1)
                $children = Array($children);
            else
                $children = Array();
        }
        foreach ($children as $child) {
            $thumb = (is_object($child->thumb) ? $child->thumb->fid : $child->thumb);
            if (strlen($child->title) > 20)
                $title = substr($child->title, 0, 20)."...";
            else
                $title = $child->title;
            $options[$thumb] = $title;
        }
        $output .= "<br/>\n";
        $output .= form_select(t("Album thumbnail"), "{$name_prefix}thumb", ($mythumb?$mythumb:'0'), $options, NULL, "onchange='set_thumb(this, {$node->nid}, ".count($children).");'");
        break;
    }
    return $output;
}

function _class_album_destroy(&$node) {
    // delete all children
    $kids = _acidfree_get_children($node);
    foreach ($kids as $kid) {
        if ($kid->class != 'album' && count($kid->parent) > 1) {
            // delete from this album only
            $key = array_search($node->nid, $kid->parent);
            unset($n->parent[$key]);
            node_save($kid);
        } else {
            node_delete(array('nid' => $kid->nid, 'confirm' => 1));
        }
    }
}

function _album_create_links(&$node) {
    global $acidfree_types;
    $links = Array();
    foreach ($acidfree_types as $type) {
        if (user_access($type->access)){
          if(preg_match('/^[aeiou]/',$type->class)){ 
            $links[] = l(t('Add an %type', array('%type' => $type->class)),
                    "node/add/acidfree/{$type->class}/{$node->nid}");
          }
          else
          {
            $links[] = l(t('Add a %type', array('%type' => $type->class)),
                    "node/add/acidfree/{$type->class}/{$node->nid}");            
          }
        }
    }
    if (user_access('acidfree mass import'))
        $links[] = l(t('Mass import'), "node/add/acidfree/mass/{$node->nid}");
    return implode("<br/>\n", $links);
}

function _acidfree_get_album_thumb(&$node) {
    $query = "SELECT n.nid, child FROM {node} n ".
      "INNER JOIN {acidfree} ON n.nid = {acidfree}.aid ".
      "INNER JOIN {acidfree_hierarchy} ON {acidfree}.aid = {acidfree_hierarchy}.child ".
      "WHERE {acidfree_hierarchy}.parent = '%s' AND {acidfree}.class %s 'album' ".
      "ORDER BY RAND() LIMIT 1";
    $result = db_query(db_rewrite_sql($query), $node->nid, '<>');

    if (db_num_rows($result) != 1) {
        // Re-use query from above except change operation from <> to =
        $result = db_query(db_rewrite_sql($query), $node->nid, '=');
        if (db_num_rows($result) != 1)
            return '';
        $row = db_fetch_array($result);
        return _acidfree_get_album_thumb(acidfree_get_node_by_id($row[child]));
    } else {
        $thumb = db_fetch_array($result);
        $thumb = acidfree_get_node_by_id($thumb['child']);
        $thumb = _acidfree_get_thumb_url($thumb);
    }   
    return $thumb;
}

function theme_acidfree_print_full_album($node) {
    $ncols = variable_get('acidfree_cols', 5);
    $nrows = variable_get('acidfree_rows', 3);
    $show_box = node_access('update', $node);
    $nodes_per_page = $ncols*$nrows - ($show_box ? 1 : 0);
    $nodes = _acidfree_get_children($node, $nodes_per_page, ALBUM_PAGER);
    if (count($nodes) == 0) 
        return "This album is empty<br>".($show_box?_album_create_links($node):"");
    if ($node->page) {
        $output .= theme('pager', NULL, $nodes_per_page, ALBUM_PAGER);
        if ($show_box) {
            $row[] = Array('data' => _album_create_links($node), 'class' => 'album-cell');
            $count++;
        }
    }
    foreach ($nodes as $child_node) {
        $row[] = Array(
                'data' => theme("acidfree_print_thumb_{$child_node->class}",
                                $child_node, $node),
                'class' => 'album-cell');
        if (++$count % $ncols == 0) {
            $rows[] = Array('data' => $row, 'class' => 'none');
            unset($row);
        }
    }
    if (isset($row))
        $rows[] = Array('data' => $row, "class" => "none");
    $output .= theme('table', NULL, $rows, Array('id' => 'acidfree-main'), false);
    $output .= "<div class='acidfree-body'>{$node->body}</div>";
    return $output;
}

function theme_acidfree_print_thumb_album(&$node) {
    $album_link = "node/{$node->nid}";
    $image = _acidfree_get_thumb_url($node);
    if ($image != '') {
        $image = l(theme('image', $image, $node->title, $node->title, 
                    "class='album-thumb' id='acidfree-thumb{$node->nid}'", false),
                $album_link, array(), NULL, NULL, true, true);
    }
    $size = variable_get('acidfree_thumb_dim', IMAGE_THUMB_SIZE) + 40;
    $output .= "<div class='fullslide' style='width: {$size}px;'>\n";
    $output .= "<div class='folder' onclick='document.location=\"".url($album_link, null, null, true)."\"'>";
    // XXX: this empty span is to make IE render vertical alignment correctly
    $output .= "<span></span>\n";
    $output .= $image;
    $output .= '</div><div>'.$node->title;
    //$output .= '</div><div>'.l($node->title, $album_link);
    $output .= "</div></div>\n";
    return $output;
}

?>
