<?php
/* $Id: class_photo.inc,v 1.2.2.24 2005/12/19 15:41:51 vhmauery Exp $ */

/*
Acidfree Photo Albums for Drupal
Copyright (C) 2005 Vernon Mauery

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/

/**
 * @file
 * The is the class implementation of the photo.  All the acidfree elements
 * must be defined in this same manner, which makes creating a new class
 * relatively simple -- copy one of the existing classes and modify the 
 * relevant functions.
 */

require_once('image_manip.inc');

function class_photo_register() {
    $class = new stdClass();
    $class->class = 'photo';
    $class->max_size = 2*(1 << 20);
    /* i.e. reports image/pjpeg as mime type for jpegs.  grrr. */
    $class->mime_ext = Array('image/jpeg'=>'jpg', 'image/pjpeg'=>'jpg', 'image/png'=>'png', 'image/gif' => 'gif');
    $class->create = '_class_photo_create';
    $class->update = '_class_photo_update';
    $class->form = '_class_photo_form';
    $class->destroy = '_class_photo_destroy';
    $class->access = 'create acidfree photos';
    $class->resize = '_class_photo_resize';
    return $class;
}

function _class_photo_create(&$node, &$file) {
    global $acidfree_types;
    $ext = $acidfree_types['photo']->mime_ext[$file->filemime];
    $images = (variable_get('acidfree_large_dim', -1) == 0) ?
        Array('small', 'thumb'):
        Array('large', 'small', 'thumb');
    foreach ($images as $size) {
      $node->$size = acidfree_add_file($node, $file->filepath, "{$node->nid}_{$size}.{$ext}", $file->filemime, false);
    }
    if ($file->source != 'acidfree_local_filesystem') {
        file_delete($file->filepath);
    }
    _class_photo_resize($node);
}

function _class_photo_resize(&$node) {
    $largefile = _acidfree_get_large_path($node, true);
    $smallfile = _acidfree_get_small_path($node, true);
    $thumbfile = _acidfree_get_thumb_path($node, true);
    if (($size = variable_get('acidfree_large_dim', -1)) > 0 && $largefile) {
        $info = getimagesize($largefile);
        // give them a fudge factor for megapixel size -- up to .5 M pixels bigger
        if ($info[0] * $info[1] > ($size + 500000)) {
            $size = sqrt($size * ($info[0]/$info[1]));
            acidfree_resize_image($largefile, $largefile, $size);
        }
    } else if ($size == 0 && $largefile) {
        module_invoke('filemanager', 'delete', $node->large);
        db_query("UPDATE {acidfree} SET large='' WHERE aid={$node->nid}");
        $node->large = '';
    }
    $size = variable_get('acidfree_small_dim', IMAGE_SMALL_SIZE);
    if (is_file($smallfile)) {
        $info = getimagesize($smallfile);
    }
    if (max($info[0], $info[1]) != $size) {
        $source = ($largefile ? $largefile : $smallfile);
        acidfree_resize_image($source, $smallfile, $size);
    }
    $size = variable_get('acidfree_thumb_dim', IMAGE_THUMB_SIZE);
    if (is_file($thumbfile))
        $info = getimagesize($thumbfile);
    if (max($info[0], $info[1]) != $size) {
        acidfree_resize_image($smallfile, $thumbfile, $size);
    }
    return true;
}

function _class_photo_update(&$node) {
    if ($node->rotate != 0) {
        $large = module_invoke('filemanager', 'get_working_copy', $node->large, true);
        $large_path = module_invoke('filemanager', 'create_path', $large, true);
        $small = module_invoke('filemanager', 'get_working_copy', $node->small, true);
        $small_path = module_invoke('filemanager', 'create_path', $small, true);
        $thumb = module_invoke('filemanager', 'get_working_copy', $node->thumb, true);
        $thumb_path = module_invoke('filemanager', 'create_path', $thumb, true);
        acidfree_rotate_image($large_path, $node->rotate);
        acidfree_rotate_image($small_path, $node->rotate);
        // do not try rotating the thumb -- it gets too distorted -- resample it
        if (!acidfree_resize_image($small_path, $thumb_path,
                    variable_get('acidfree_thumb_dim', IMAGE_THUMB_SIZE))) {
            $node->thumb = $node->small;
        } else {
        }
        module_invoke('filemanager', 'promote_working', $large);
        module_invoke('filemanager', 'promote_working', $small);
        module_invoke('filemanager', 'promote_working', $thumb);
    }
    return null;
}

function _class_photo_form($op, &$node, $name_prefix='') {
    $output = '';
    switch ($op) {
    case 'new':
        break;
    case 'preview':
        $output .= form_hidden('thumb', is_object($node->thumb)?$node->thumb->fid:$node->thumb);
        break;
    case 'update':
        $output .= form_hidden('thumb', is_object($node->thumb)?$node->thumb->fid:$node->thumb);
        $options = Array('270' => t('Counterclockwise 90%deg', array('%deg' => '&deg;')), '0' => t('No rotation'), '90' => t('Clockwise 90%deg', array('%deg' => '&deg;')), 'auto'=>t('Use exif orientation tag'));
        $output .= form_radios(t('Rotate image'), "{$name_prefix}rotate", '0', $options);
        break;
    }
    return $output;
}

function _class_photo_destroy(&$node) {
}

function theme_acidfree_print_full_photo(&$node) {
    if (!$node->small)
        return '';
    if ($node->page)
        $output .= acidfree_pager_creator($node);

    $output .= "<div class='acidfree-full'>";
    $largeurl = _acidfree_get_large_url($node);
    if (!$largeurl) {
        $output .= theme('image', _acidfree_get_small_url($node),
                $node->title, $node->title, NULL, false);
    } else {
        $output .= "<a href='"._acidfree_get_large_url($node)."'>".
            theme('image', _acidfree_get_small_url($node),
                    $node->title, $node->title, NULL, false).
            "</a>";
    }
    $output .= "</div><div class='acidfree-body'>{$node->body}</div>";
    if (variable_get('acidfree_show_exif_data', false)) {
        $output .= theme('acidfree_exif_data', $node);
    }

    return $output;
}

function theme_acidfree_print_thumb_photo(&$node, $parent=null) {
    if (count($node->parent) > 1 && $parent)
        $p = "pid={$parent->nid}";

    $image = l(theme('image', _acidfree_get_thumb_url($node),
                $node->title, $node->title, "class='acidfree'", false),
              "node/{$node->nid}", array(), $p, NULL, true, true);
    $size = variable_get('acidfree_thumb_dim', IMAGE_THUMB_SIZE) + 40;
    $cell .= "<div class='fullslide' style='width: {$size}px;'>\n";
    $size -= 20;
    $cell .= "<div class='outer' style='width: {$size}px; height: {$size}px;'>\n";
    // XXX: this empty span is to make IE render vertical alignment correctly
    $cell .= "<span></span>\n";
    $cell .= "$image\n";
    $cell .= "</div>\n";
    $cell .= "<div>{$node->title}</div>\n";
    $cell .= "</div>\n";
    return $cell;
}

function theme_acidfree_exif_data(&$node) {
    $output = '<div id="acidfree-exif-data"><h3>'.t('EXIF data:').'</h3>';
    $largefile = _acidfree_get_large_path($node);
    if ($largefile) {
        $exif = exif_read_data($largefile, 0, true);
        foreach ($exif as $key => $section) {
            foreach ($section as $name => $val) {
                $output .= "$key.$name: $val<br />\n";
            }
        }
    } else {
        $output .= t('no exif data');
    }
    $output .= '</div>';
    return $output;
}

?>
