<?php
/* $Id: class_video.inc,v 1.2.2.22 2006/04/25 18:16:42 vhmauery Exp $ */

/*
Acidfree Photo Albums for Drupal
Copyright (C) 2005 Vernon Mauery

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/

/**
 * @file
 * The is the class implementation of the photo.  All the acidfree elements
 * must be defined in this same manner, which makes creating a new class
 * relatively simple -- copy one of the existing classes and modify the 
 * relevant functions.
 */

require_once('image_manip.inc');

function class_video_register() {
    $class = new stdClass();
    $class->class = 'video';
    $class->max_size = 4*(1 << 20);
    $class->mime_ext = Array(
            'video/x-msvideo'=>'avi',
            'video/mpeg'=>'mpg',
            'video/mp4'=>'mp4',
            'video/quicktime'=>'mov',
            'audio/x-pn-realaudio'=>'rm',
            'audio/x-realaudio'=>'rm',
            'application/vnd.rn-realmedia'=>'rm',
            'video/x-ms-wmv'=>'wmv',
            'video/x-ms-asf'=>'asf',
            'video/3gpp'=>'3gp',
        );
    $class->create = '_class_video_create';
    $class->update = '_class_video_update';
    $class->form = '_class_video_form';
    $class->destroy = '_class_video_destroy';
    $class->access = 'create acidfree videos';
    $class->resize = '_class_video_resize';
    return $class;
}

function _class_video_create(&$node, &$file) {
    global $acidfree_types;
    $mime = 'image/jpeg';
    $ext = 'jpg';
    $node->thumb = acidfree_add_file($node, acidfree_mktmpfile('videosmall'), "{$node->nid}_thumb.{$ext}", $mime, true);
    $method = variable_get('acidfree_video_thumbnailer', 'none');
    $tfile = file_check_upload('userthumb');
    if ($tfile) {
        $node->small = acidfree_add_file($node, $tfile->filepath, "{$node->nid}_small.{$ext}", $mime, true);
    } else {
        $node->small = acidfree_add_file($node, acidfree_mktmpfile('videosmall'), "{$node->nid}_small.{$ext}", $mime, true);
    }
    $mime = $file->filemime;
    $ext = $acidfree_types['video']->mime_ext[$mime];
    $node->large = acidfree_add_file($node, $file->filepath, "{$node->nid}_large.{$ext}", $mime, false);

    // TODO: in later versions we might want to support having more than one resolution
    // but for now, the small and large are the same fid
    if ($file->source != 'acidfree_local_filesystem')
        file_delete($file->filepath);
    _class_video_resize($node);
}

function _class_video_resize(&$node) {
    $acidfree_base = drupal_get_path('module', 'acidfree');
    $largefile = _acidfree_get_large_path($node, true);
    $smallfile = _acidfree_get_small_path($node, true);
    $thumbfile = _acidfree_get_thumb_path($node, true);
    $method = variable_get('acidfree_video_thumbnailer', 'none');
    // in the case of mass import, we go mplayer or none
    if ($method == 'user' && filesize($smallfile) == 0)
        $method = 'none';
    // we aren't required to resize the small image because it holds our video size
    if (filesize($smallfile) == 0) {
        switch ($method) {
        case 'none':
            acidfree_resize_image($acidfree_base.DIRECTORY_SEPARATOR.
                        'art'.DIRECTORY_SEPARATOR.'video.jpg', $smallfile,
                $node->videox, $node->videoy);
            break;
        case 'mplayer':
            $tmpdir = acidfree_mktmpdir('mplayer');
            if (strchr($tmpdir, ':')) {
                /* XXX: the chdir/getcwd crap is because mplayer doesn't know how
                 *      to deal with the c:\ at the front of a windows path when put
                 *      in the suboption list for -vo jpeg */
                $oldcwd = getcwd();
                if (!chdir($tmpdir)) {
                    drupal_set_message("could not change dir to temp dir", 'error');
                }
                $tmpdirarg = '';
            } else {
                $tmpdirarg = ":outdir=$tmpdir";
            }
            $mplayer = variable_get('acidfree_path_to_mplayer', '/usr/bin/mplayer');
            $command = "$mplayer -ao null -vo jpeg:maxfiles=1:quality=100{$tmpdirarg} -frames 1 $largefile";
            //echo "$command<br>\n";
            $mplayeroutput = shell_exec($command);
            //echo "$mplayeroutput<br><br>\n";
            if ($oldcwd) {
                chdir($oldcwd);
            }
            $mplayer_out = $tmpdir.DIRECTORY_SEPARATOR."00000001.jpg";
            if (!file_exists($mplayer_out)) {
                drupal_set_message("file $mplayer_out does not exist", 'error');
                //lsfiles(dirname(_acidfree_lockfile()));
            } else {
                copy($mplayer_out, $smallfile);
                // lsfiles(dirname(_acidfree_lockfile()));
            }
            rmdir_rec($tmpdir);
            break;
        case 'user':
            break;
        }
    }
    $size = variable_get('acidfree_thumb_dim', IMAGE_THUMB_SIZE);
    if (is_file($thumbfile) && filesize($thumbfile) > 0) {
        $info = getimagesize($thumbfile);
    }
    if (max($info[0], $info[1]) != $size) {
        acidfree_resize_image($smallfile, $thumbfile, $size);
    }
    return true;
}

function _class_video_update(&$node) {
}

function _class_video_form($op, &$node, $name_prefix='') {
    switch ($op) {
    case 'new':
        $nailer = variable_get('acidfree_video_thumbnailer', 'none');
        $output = '';
        switch ($nailer) {
        case 'none':
            $output .= form_textfield(t('Video width'), $name_prefix.'videox', 320, 10, 10);
            $output .= form_textfield(t('Video height'), $name_prefix.'videoy', 240, 10, 10);
            break;
        case 'user':
            $output .= form_file(t('Video thumbnail'), $name_prefix.'userthumb', 50, NULL, true);
            break;
        case 'mplayer':
            // possibly allow them to choose frame number? too complex?
            // too much decoding time -- no option to tell where to start decoding...
            break;
        }
        break;
    case 'preview':
        $output .= form_hidden('thumb', is_object($node->thumb)?$node->thumb->fid:$node->thumb);
        break;
    case 'update':
        $output .= form_hidden('thumb', is_object($node->thumb)?$node->thumb->fid:$node->thumb);
        break;
    }
    return $output;
}

function _class_video_destroy(&$node) {
}

function _class_video_mplayer_options() {
    $name = 'acidfree_path_to_mplayer';
    $mplayer_path = variable_get($name, '/usr/bin/mplayer');
    if (!is_executable($mplayer_path))
        form_set_error($name, t('Path to mplayer is not correct &mdash; video thumbnails will not be generated'));
    return  form_textfield(t('Path to mplayer'), $name, $mplayer_path, 64, 64);
}
function _class_video_manipulation_options() {
    $options = array(
            'none'=>t('No thumbnail &mdash; use video image'),
            'user'=>t('User uploaded thumbnails'),
            'mplayer'=>t('Use mplayer to create thumbnails')
    );
    $name = 'acidfree_video_thumbnailer';
    $value = variable_get($name, 'none');
    return form_radios(t('Video thumbnail options'), $name, $value, $options).
        acidfree_call("_class_video_{$value}_options");
}

function theme_acidfree_print_full_video(&$node) {
    if (!$node->large)
        return '';

    // get the pager
    if ($node->page)
        $output = acidfree_pager_creator($node);

    $id = $node->nid;
    $name = $node->title;

    // icon link to the full size video
    $full_video = _acidfree_get_large_url($node);
    $output .= "<a href='$full_video'>".t('download video').'</a><br/>';

    $ext = pathinfo(_acidfree_get_large_path($node), PATHINFO_EXTENSION);
    switch ($ext) {
    case 'rm':
        $type = 'realmedia';
        break;
    case 'mp4':
    case 'mov':
        $type = 'quicktime';
        break;
    default:
        $type = 'windowsmedia';
        break;
    }
    $finfo = image_get_info(_acidfree_get_small_path($node, true));
    $node->videoy = $finfo['height'];
    $node->videox = $finfo['width'];
    $output .= acidfree_call("theme_video_{$type}_control",Array(&$node));
    $output .= "<div class='acidfree-body'>{$node->body}</div>\n";

    return $output;
}

function theme_acidfree_print_thumb_video(&$node, $parent=null) {
    if (count($node->parent) > 1 && $parent)
        $p = "pid={$parent->nid}";

    $image = l(theme('image', _acidfree_get_thumb_url($node),
                $node->title, $node->title, "class='acidfree'", false),
              "node/{$node->nid}", array(), $p, NULL, true, true);
    $size = variable_get('acidfree_thumb_dim', IMAGE_THUMB_SIZE) + 40;
    $cell .= "<div class='fullslide' style='width: {$size}px;'>\n";
    $size -= 20;
    $cell .= "<div class='outer video' style='width: {$size}px; height: {$size}px;'>\n";
    // XXX: this empty span is to make IE render vertical alignment correctly
    $cell .= "<span class='iefiller'></span>\n";
    $cell .= "$image";
    $cell .= "</div>\n";
    $cell .= "<div>{$node->title}</div>\n";
    $cell .= "</div>\n";
    return $cell;
}

/*********************************************************************
 * Themeable functions for playing videos.
 * It prints a page with a player embedded
 * linked to the file record of the node.
 * 
 * this code was taken from the video.module because they did the 
 * drawing a LOT better than I was.  I modified it a bit too so 
 * the drawing routines don't call theme_page, that is node_view's job.
 *********************************************************************/

function _video_view_help($text, $link, $alt) {
    $output = "<p>".t('Problems viewing videos?')."<br />\n";
    $output .= l($text, $link, Array('title'=>$alt));
    $output .= "</p>\n";
    return $output;
}

/**
 * Play videos from in Quicktime format
 *
 * @param $node
 *   object with node information
 *
 * @return
 *   string of content to display
 */
function theme_video_quicktime_control(&$node) {
    //Increase the height to accommodate the player controls on the bottom.
    $height = $node->videoy + 16;
    $vidfile = _acidfree_get_large_url($node);
    $output .= '<object classid="clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B" width="'.$node->videox.'" height="'.$height.'" scale="tofit" codebase="http://www.apple.com/qtactivex/qtplugin.cab">          
        <param name="SRC" value="'.$vidfile.'" />
        <param name="AUTOPLAY" value="true" />
        <param name="KIOSKMODE" value="false" />
        <embed src="'.$vidfile.'" width="'.$node->videox.'" height="'.$height.'" scale="tofit" autoplay="true" kioskmode="false" pluginspage="http://www.apple.com/quicktime/download/">
        </embed>
        </object>';
    if ($node->page)
        $output .= _video_view_help(t('Download latest Quicktime Player'), 'http://www.apple.com/quicktime/download', t('Link to QuickTime Download Page'));
    return $output;
}

/**
 * Play videos from in Realmedia format
 *
 * @param $node
 *   object with node information
 *
 * @return
 *   string of content to display
 */
function theme_video_realmedia_control(&$node) {
    // Real's embeded player includes the controls
    // in the height
    $height = $node->videoy + 40;
    $vidfile = _acidfree_get_large_url($node);
    $output .= '<object id="video1" CLASSID="clsid:CFCDAA03-8BE4-11cf-B84B-0020AFBBCCFA" height="'.$height.'" width="'.$node->videox.'">
        <param name="_ExtentX" value="7276">
        <param name="_ExtentY" value="3307">
        <param name="AUTOSTART" value="1">
        <param name="SHUFFLE" value="0">
        <param name="PREFETCH" value="0">
        <param name="NOLABELS" value="0">
        <param name="SRC" value="'.$vidfile.'" ref>
        <param name="CONTROLS" value="All">
        <param name="CONSOLE" value="Clip1">
        <param name="LOOP" value="0">
        <param name="NUMLOOP" value="0">
        <param name="CENTER" value="0">
        <param name="MAINTAINASPECT" value="0">
        <param name="BACKGROUNDCOLOR" value="#000000">
        <embed src="'.$vidfile.'" type="audio/x-pn-realaudio-plugin" console="Clip1" controls="All" height="'.$height.'" width="'.$node->videox.'" autostart="true">
        </object>';
    if ($node->page)
        $output .= _video_view_help(t('Download latest Realmedia Player'), 'http://www.real.com/', t('Link to Real'));
    return $output;
}

/**
 * Play videos from in WindowsMediaVideo format
 *
 * @param $node
 *   object with node information
 *
 * @return
 *   string of content to display
 */
function theme_video_windowsmedia_control(&$node) {
    // Windows Media's embeded player includes the controls in the height
    $height = $node->videoy + 68;
    $vidfile = _acidfree_get_large_url($node);
    $output .= '<OBJECT id="video1" width="'.$node->videox.'" height="'.$height.'"
        classid="CLSID:22d6f312-b0f6-11d0-94ab-0080c74c7e95" 
        codebase="http://activex.microsoft.com/activex/controls/mplayer/en/nsmp2inf.cab#Version=5,1,52,701"
        standby="Loading Microsoft Windows Media Player components..." type="application/x-oleobject">
        <param name="fileName" value="'.$vidfile.'">
        <param name="animationatStart" value="true">
        <param name="transparentatStart" value="true">
        <param name="autoStart" value="true">
        <param name="showControls" value="true">
        <param name="loop" value="true">
        <EMBED type="application/x-mplayer2"
        pluginspage="http://microsoft.com/windows/mediaplayer/en/download/"
        id="mediaPlayer" name="mediaPlayer" displaysize="4" autosize="-1" 
        showcontrols="true" showtracker="-1" 
        showdisplay="0" showstatusbar="-1" videoborder3d="-1" width="'.$node->videox.'" height="'.$height.'"
        src="'.$vidfile.'" autostart="true" loop="true">
        </EMBED>
        </OBJECT>';

    if ($node->page)
        $output .= _video_view_help(t('Download latest Windows Media Player'),
            t('http://windowsupdate.microsoft.com/'),
            t('Link to Windows Update'));
    return $output;
}

?>
